@@ -29,7 +29,8 @@
 
 /* Handy macros */
 #define QF_IS_OLD(qf)		(qf & (1 << QF_VFSOLD))
-#define QF_IS_NEW(qf)		(qf & (1 << QF_VFSV0))
+#define QF_IS_V0(qf)		(qf & (1 << QF_VFSV0))
+#define QF_IS_V1(qf)		(qf & (1 << QF_VFSV1))
 #define QF_IS_XFS(qf)		(qf & (1 << QF_XFS))
 #define QF_IS_TOO_NEW(qf)	(qf == QF_TOONEW)
 #define IF_GENERIC		(kernel_iface == IFACE_GENERIC)
@@ -37,15 +38,14 @@
 static int quota_format;
 static int kernel_iface;
 
-static int v1_quota_get(quota_t *);
-static int v1_quota_set(quota_t *);
-static int v2_quota_get(quota_t *);
-static int v2_quota_set(quota_t *);
+static int old_quota_get(quota_t *);
+static int old_quota_set(quota_t *);
+static int v0_quota_get(quota_t *);
+static int v0_quota_set(quota_t *);
 static int generic_quota_get(quota_t *);
 static int generic_quota_set(quota_t *);
 static int xfs_quota_get(quota_t *);
 static int xfs_quota_set(quota_t *);
-int xfs_reset_grace(quota_t *, int);
 
 quota_t *quota_new (int q_type, int id, char *fs_spec)
 {
@@ -75,7 +75,7 @@ quota_t *quota_new (int q_type, int id, char *fs_spec)
    */
   output_debug("Detecting quota format");
   if (kern_quota_format(fs, q_type) == QF_ERROR) {
-     output_error("Can't determine quota format!");
+     output_error("Cannot determine quota format!");
      exit (ERR_SYS);
   }
   if (QF_IS_TOO_NEW(quota_format)) {
@@ -85,20 +85,30 @@ quota_t *quota_new (int q_type, int id, char *fs_spec)
   if (QF_IS_XFS(quota_format)) {
      output_debug("Detected quota format: XFS");
   }
-  if (QF_IS_NEW(quota_format)) {
-     output_debug("Detected quota format: NEW");
+  if (QF_IS_V0(quota_format)) {
+     output_debug("Detected quota format: VFSV0");
      if (IF_GENERIC) {
        output_debug("Detected quota interface: GENERIC");
      }
      else {
-       myquota->_v2_quotainfo = (struct v2_kern_dqinfo *) 0;
-       myquota->_v2_quotainfo = (struct v2_kern_dqinfo *) malloc (sizeof(struct v2_kern_dqinfo));
-       if ( ! myquota->_v2_quotainfo ) {
+       myquota->_v0_quotainfo = (struct v0_kern_dqinfo *) 0;
+       myquota->_v0_quotainfo = (struct v0_kern_dqinfo *) malloc (sizeof(struct v0_kern_dqinfo));
+       if ( ! myquota->_v0_quotainfo ) {
 	 output_error ("Insufficient memory");
 	 exit (ERR_MEM);
        }
      }
   }
+  else if (QF_IS_V1(quota_format)) {
+     output_debug("Detected quota format: VFSV1");
+     if (IF_GENERIC) {
+       output_debug("Detected quota interface: GENERIC");
+     }
+     else {
+       output_error("Unsupported quota format: VFSV1 but not GENERIC, please report Issue on github: https://github.com/ekenberg/quotatool");
+       exit(ERR_SYS);
+     }
+  }
   else if (QF_IS_OLD(quota_format)) {
      output_debug("Detected quota format: OLD");
      if (IF_GENERIC) {
@@ -118,7 +128,7 @@ quota_t *quota_new (int q_type, int id, char *fs_spec)
        }
   }
 
-  qfile = strdup (fs->device);  
+  qfile = strdup (fs->device);
 
   myquota->_id = id;
   myquota->_id_type = q_type;
@@ -133,10 +143,10 @@ inline void quota_delete (quota_t *myquota) {
   if (IF_GENERIC) {
     free (myquota->_generic_quotainfo);
   }
-  else if (QF_IS_NEW(quota_format)) {
-     free (myquota->_v2_quotainfo);
+  else if (QF_IS_V0(quota_format)) {
+     free (myquota->_v0_quotainfo);
   }
-  
+
   free (myquota);
 }
 
@@ -151,41 +161,37 @@ int quota_get (quota_t *myquota) {
    else if (IF_GENERIC) {
       retval = generic_quota_get(myquota);
    }
-   else if (QF_IS_NEW(quota_format)) {
-      retval = v2_quota_get(myquota);
+   else if (QF_IS_V0(quota_format)) {
+      retval = v0_quota_get(myquota);
    }
    else {
-      retval = v1_quota_get(myquota);
+      retval = old_quota_get(myquota);
    }
    return retval;
 }
 
-static int v1_quota_get (quota_t *myquota) {
-  struct v1_kern_dqblk sysquota;
+static int old_quota_get (quota_t *myquota) {
+  struct old_kern_dqblk sysquota;
   int retval;
 
-  retval = quotactl(QCMD(Q_V1_GETQUOTA,myquota->_id_type), myquota->_qfile,
+  retval = quotactl(QCMD(Q_OLD_GETQUOTA,myquota->_id_type), myquota->_qfile,
 		    myquota->_id, (caddr_t) &sysquota);
   if ( retval < 0 ) {
-    output_error ("Failed fetching quotas (v1): %s", strerror(errno));
+    output_error ("Failed fetching quotas (old): %s", strerror(errno));
     return 0;
   }
 
   /* copy the linux-formatted quota info into our struct */
   myquota->block_hard  = sysquota.dqb_bhardlimit;
   myquota->block_soft  = sysquota.dqb_bsoftlimit;
-  // I believe this might be wrong:
-  // dqb_curblocks should probably be multiplied by the block size of the filesystem
-  // since myquota->block_used is really a byte-count...
-  // (yeah, must change the field name in the struct)
-  myquota->block_used  = sysquota.dqb_curblocks;
+  myquota->diskspace_used  = sysquota.dqb_curblocks;
   myquota->inode_hard  = sysquota.dqb_ihardlimit;
   myquota->inode_soft  = sysquota.dqb_isoftlimit;
   myquota->inode_used  = sysquota.dqb_curinodes;
   myquota->block_time  = sysquota.dqb_btime;
   myquota->inode_time  = sysquota.dqb_itime;
-  /* yes, something fishy here. quota v1 seems to lack separate fields
-   for user grace times and global grace times. 
+  /* yes, something fishy here. quota old seems to lack separate fields
+   for user grace times and global grace times.
   is it like XFS - root's limits sets global? */
   myquota->block_grace  = sysquota.dqb_btime;
   myquota->inode_grace  = sysquota.dqb_itime;
@@ -193,36 +199,36 @@ static int v1_quota_get (quota_t *myquota) {
   return 1;
 }
 
-static int v2_quota_get (quota_t *myquota) {
-  struct v2_kern_dqblk sysquota;
+static int v0_quota_get (quota_t *myquota) {
+  struct v0_kern_dqblk sysquota;
   int retval;
 
-  retval = quotactl(QCMD(Q_V2_GETQUOTA,myquota->_id_type), myquota->_qfile,
+  retval = quotactl(QCMD(Q_V0_GETQUOTA,myquota->_id_type), myquota->_qfile,
 		    myquota->_id, (caddr_t) &sysquota);
   if ( retval < 0 ) {
-    output_error ("Failed fetching quotas (v2): %s", strerror(errno));
+    output_error ("Failed fetching quotas (vfsv0): %s", strerror(errno));
     return 0;
   }
 
   /* copy the linux-formatted quota info into our struct */
   myquota->block_hard = sysquota.dqb_bhardlimit;
   myquota->block_soft = sysquota.dqb_bsoftlimit;
-  myquota->block_used = sysquota.dqb_curspace;
+  myquota->diskspace_used = sysquota.dqb_curspace;
   myquota->inode_hard = sysquota.dqb_ihardlimit;
   myquota->inode_soft = sysquota.dqb_isoftlimit;
   myquota->inode_used = sysquota.dqb_curinodes;
   myquota->block_time = sysquota.dqb_btime;
   myquota->inode_time = sysquota.dqb_itime;
 
-  retval = quotactl(QCMD(Q_V2_GETINFO,myquota->_id_type), myquota->_qfile,
-		    myquota->_id, (caddr_t) myquota->_v2_quotainfo);
+  retval = quotactl(QCMD(Q_V0_GETINFO,myquota->_id_type), myquota->_qfile,
+		    myquota->_id, (caddr_t) myquota->_v0_quotainfo);
   if ( retval < 0 ) {
     output_error ("Failed fetching quotainfo: %s", strerror(errno));
     return 0;
   }
-  myquota->block_grace = ((struct v2_kern_dqinfo *) myquota->_v2_quotainfo)->dqi_bgrace;
-  myquota->inode_grace = ((struct v2_kern_dqinfo *) myquota->_v2_quotainfo)->dqi_igrace;
-  
+  myquota->block_grace = ((struct v0_kern_dqinfo *) myquota->_v0_quotainfo)->dqi_bgrace;
+  myquota->inode_grace = ((struct v0_kern_dqinfo *) myquota->_v0_quotainfo)->dqi_igrace;
+
   return 1;
 }
 
@@ -239,7 +245,7 @@ static int generic_quota_get (quota_t *myquota) {
   /* copy the linux-formatted quota info into our struct */
   myquota->block_hard = sysquota.dqb_bhardlimit;
   myquota->block_soft = sysquota.dqb_bsoftlimit;
-  myquota->block_used = sysquota.dqb_curspace;
+  myquota->diskspace_used = sysquota.dqb_curspace;
   myquota->inode_hard = sysquota.dqb_ihardlimit;
   myquota->inode_soft = sysquota.dqb_isoftlimit;
   myquota->inode_used = sysquota.dqb_curinodes;
@@ -254,7 +260,7 @@ static int generic_quota_get (quota_t *myquota) {
   }
   myquota->block_grace = ((struct if_dqinfo *) myquota->_generic_quotainfo)->dqi_bgrace;
   myquota->inode_grace = ((struct if_dqinfo *) myquota->_generic_quotainfo)->dqi_igrace;
-  
+
   return 1;
 }
 
@@ -267,7 +273,7 @@ static int xfs_quota_get(quota_t *myquota) {
    block_diff = BLOCK_SIZE / 512;
    retval = quotactl(QCMD(Q_XGETQUOTA, myquota->_id_type), myquota->_qfile,
 		     myquota->_id, (caddr_t) &sysquota);
-   /* 
+   /*
       ** 2005-04-26  : fmicaux@actilis.net -
                         handling a non-set quota for a user/group
                                              who owns nothing here
@@ -278,16 +284,16 @@ static int xfs_quota_get(quota_t *myquota) {
       // but at this point of the code, whe know that this XFS has quotas.
       // We make the choice to produce a "0 0 0 0 0 0 0 0" line.
       if ( errno == ENOENT ) {
-         myquota->block_hard	= 0;
-         myquota->block_soft	= 0;
-         myquota->block_used	= 0;
-         myquota->inode_hard    = 0;
-         myquota->inode_soft    = 0;
-         myquota->inode_used    = 0;
-         myquota->block_grace	= 0;
-         myquota->inode_grace	= 0;
-         myquota->block_time	= 0;
-         myquota->inode_time	= 0;
+         myquota->block_hard 	 = 0;
+         myquota->block_soft	 = 0;
+         myquota->diskspace_used = 0;
+         myquota->inode_hard     = 0;
+         myquota->inode_soft     = 0;
+         myquota->inode_used     = 0;
+         myquota->block_grace	 = 0;
+         myquota->inode_grace	 = 0;
+         myquota->block_time	 = 0;
+         myquota->inode_time	 = 0;
          return 1;
       }
       output_error ("Failed fetching quotas: errno=%d, %s", errno, strerror(errno));
@@ -298,16 +304,17 @@ static int xfs_quota_get(quota_t *myquota) {
 		     myquota->_id, (caddr_t) &quotastat);
 
    /* copy the linux-xfs-formatted quota info into our struct */
-   myquota->block_hard	= sysquota.d_blk_hardlimit / block_diff;
-   myquota->block_soft	= sysquota.d_blk_softlimit / block_diff;
-   myquota->block_used	= sysquota.d_bcount / block_diff;
-   myquota->inode_hard  = sysquota.d_ino_hardlimit;
-   myquota->inode_soft  = sysquota.d_ino_softlimit;
-   myquota->inode_used  = sysquota.d_icount;
-   myquota->block_grace	= quotastat.qs_btimelimit;
-   myquota->inode_grace	= quotastat.qs_itimelimit;
-   myquota->block_time	= sysquota.d_btimer;
-   myquota->inode_time	= sysquota.d_itimer;
+   myquota->block_hard	=  sysquota.d_blk_hardlimit / block_diff;
+   myquota->block_soft	=  sysquota.d_blk_softlimit / block_diff;
+ // XFS really uses blocks, all other formats in this file use bytes
+   myquota->diskspace_used = (sysquota.d_bcount * 1024) / block_diff;
+   myquota->inode_hard  =  sysquota.d_ino_hardlimit;
+   myquota->inode_soft  =  sysquota.d_ino_softlimit;
+   myquota->inode_used  =  sysquota.d_icount;
+   myquota->block_grace	=  quotastat.qs_btimelimit;
+   myquota->inode_grace	=  quotastat.qs_itimelimit;
+   myquota->block_time	=  sysquota.d_btimer;
+   myquota->inode_time	=  sysquota.d_itimer;
 
    return 1;
 }
@@ -327,11 +334,11 @@ int quota_set (quota_t *myquota){
    else if (IF_GENERIC) {
       retval = generic_quota_set(myquota);
    }
-   else if (QF_IS_NEW(quota_format)) {
-      retval = v2_quota_set(myquota);
+   else if (QF_IS_V0(quota_format)) {
+      retval = v0_quota_set(myquota);
    }
    else {
-      retval = v1_quota_set(myquota);
+      retval = old_quota_set(myquota);
    }
 
    if (! retval)
@@ -341,7 +348,7 @@ int quota_set (quota_t *myquota){
 
    /* sync */
    retval = quotactl (QCMD(IF_GENERIC ? Q_SYNC : Q_6_5_SYNC
-			   ,myquota->_id_type), myquota->_qfile, 
+			   ,myquota->_id_type), myquota->_qfile,
 		      0, NULL);
    if (retval < 0) {
       output_error ("Failed syncing quotas on %s: %s", myquota->_qfile,
@@ -358,7 +365,7 @@ static int generic_quota_set(quota_t *myquota) {
    /* copy our data into the linux dqblk */
    sysquota.dqb_bhardlimit = myquota->block_hard;
    sysquota.dqb_bsoftlimit = myquota->block_soft;
-   sysquota.dqb_curspace   = myquota->block_used;
+   sysquota.dqb_curspace   = myquota->diskspace_used;
    sysquota.dqb_ihardlimit = myquota->inode_hard;
    sysquota.dqb_isoftlimit = myquota->inode_soft;
    sysquota.dqb_curinodes  = myquota->inode_used;
@@ -368,14 +375,14 @@ static int generic_quota_set(quota_t *myquota) {
    /* make the syscall */
    retval = quotactl (QCMD(Q_SETQUOTA,myquota->_id_type),myquota->_qfile,
 		      myquota->_id, (caddr_t) &sysquota);
-   if ( retval < 0 ) { 
+   if ( retval < 0 ) {
      output_error ("Failed setting quota (generic): %s", strerror(errno));
      return 0;
    }
    /* update quotainfo (global gracetimes) */
    if (myquota->_do_set_global_block_gracetime || myquota->_do_set_global_inode_gracetime) {
      struct if_dqinfo *foo = ((struct if_dqinfo *) myquota->_generic_quotainfo);
-      if (myquota->_do_set_global_block_gracetime) 
+      if (myquota->_do_set_global_block_gracetime)
 	foo->dqi_bgrace = myquota->block_grace;
       if (myquota->_do_set_global_inode_gracetime)
 	foo->dqi_igrace = myquota->inode_grace;
@@ -390,36 +397,36 @@ static int generic_quota_set(quota_t *myquota) {
    return 1;
 }
 
-static int v2_quota_set(quota_t *myquota) {
-   struct v2_kern_dqblk sysquota;
+static int v0_quota_set(quota_t *myquota) {
+   struct v0_kern_dqblk sysquota;
    int retval;
 
    /* copy our data into the linux dqblk */
    sysquota.dqb_bhardlimit = myquota->block_hard;
    sysquota.dqb_bsoftlimit = myquota->block_soft;
-   sysquota.dqb_curspace   = myquota->block_used;
+   sysquota.dqb_curspace   = myquota->diskspace_used;
    sysquota.dqb_ihardlimit = myquota->inode_hard;
    sysquota.dqb_isoftlimit = myquota->inode_soft;
    sysquota.dqb_curinodes  = myquota->inode_used;
 //   sysquota.dqb_btime      = myquota->block_time;
 //   sysquota.dqb_itime      = myquota->inode_time;
-   
+
    /* make the syscall */
-   retval = quotactl (QCMD(Q_V2_SETQUOTA,myquota->_id_type),myquota->_qfile,
+   retval = quotactl (QCMD(Q_V0_SETQUOTA,myquota->_id_type),myquota->_qfile,
 		      myquota->_id, (caddr_t) &sysquota);
    if ( retval < 0 ) {
-      output_error ("Failed setting quota (v2): %s", strerror(errno));
+      output_error ("Failed setting quota (vfsv0): %s", strerror(errno));
       return 0;
    }
-   
+
    /* update quotainfo (global gracetimes) */
    if (myquota->_do_set_global_block_gracetime || myquota->_do_set_global_inode_gracetime) {
-      if (myquota->_do_set_global_block_gracetime) 
-	 ((struct v2_kern_dqinfo *) myquota->_v2_quotainfo)->dqi_bgrace = myquota->block_grace;
+      if (myquota->_do_set_global_block_gracetime)
+	 ((struct v0_kern_dqinfo *) myquota->_v0_quotainfo)->dqi_bgrace = myquota->block_grace;
       if (myquota->_do_set_global_inode_gracetime)
-	 ((struct v2_kern_dqinfo *) myquota->_v2_quotainfo)->dqi_igrace = myquota->inode_grace;
-      retval = quotactl (QCMD(Q_V2_SETGRACE,myquota->_id_type),myquota->_qfile,
-			 myquota->_id, (caddr_t) myquota->_v2_quotainfo);
+	 ((struct v0_kern_dqinfo *) myquota->_v0_quotainfo)->dqi_igrace = myquota->inode_grace;
+      retval = quotactl (QCMD(Q_V0_SETGRACE,myquota->_id_type),myquota->_qfile,
+			 myquota->_id, (caddr_t) myquota->_v0_quotainfo);
       if ( retval < 0 ) {
 	 output_error ("Failed setting gracetime: %s", strerror(errno));
 	 return 0;
@@ -429,26 +436,26 @@ static int v2_quota_set(quota_t *myquota) {
    return 1;
 }
 
-static int v1_quota_set(quota_t *myquota) {
-   struct v1_kern_dqblk sysquota;
+static int old_quota_set(quota_t *myquota) {
+   struct old_kern_dqblk sysquota;
    int retval;
 
   /* copy our data into the linux dqblk */
   sysquota.dqb_bhardlimit = myquota->block_hard;
   sysquota.dqb_bsoftlimit = myquota->block_soft;
-  sysquota.dqb_curblocks  = myquota->block_used;
+  sysquota.dqb_curblocks  = myquota->diskspace_used;
   sysquota.dqb_ihardlimit = myquota->inode_hard;
   sysquota.dqb_isoftlimit = myquota->inode_soft;
   sysquota.dqb_curinodes  = myquota->inode_used;
-  /* is v1 like xfs - global grace set by root's limits? */
+  /* is old like xfs - global grace set by root's limits? */
   sysquota.dqb_btime      = myquota->block_grace;
   sysquota.dqb_itime      = myquota->inode_grace;
 
   /* make the syscall */
-  retval = quotactl (QCMD(Q_V1_SETQUOTA,myquota->_id_type),myquota->_qfile,
+  retval = quotactl (QCMD(Q_OLD_SETQUOTA,myquota->_id_type),myquota->_qfile,
 		     myquota->_id, (caddr_t) &sysquota);
   if ( retval < 0 ) {
-    output_error ("Failed setting quota (v1): %s", strerror(errno));
+    output_error ("Failed setting quota (old): %s", strerror(errno));
     return 0;
   }
   /* success */
@@ -464,7 +471,8 @@ static int xfs_quota_set(quota_t *myquota) {
    /* copy our data into the linux dqblk */
    sysquota.d_blk_hardlimit = myquota->block_hard * block_diff;
    sysquota.d_blk_softlimit = myquota->block_soft * block_diff;
-   sysquota.d_bcount	    = myquota->block_used * block_diff;
+ // XFS really uses blocks, all other formats in this file use bytes
+   sysquota.d_bcount	    = DIV_UP(myquota->diskspace_used * block_diff, 1024);
    sysquota.d_ino_hardlimit = myquota->inode_hard;
    sysquota.d_ino_softlimit = myquota->inode_soft;
    sysquota.d_icount        = myquota->inode_used;
@@ -493,7 +501,7 @@ static int xfs_quota_set(quota_t *myquota) {
 
 int kern_quota_format(fs_t *fs, int q_type) {
    u_int32_t version;
-   struct v2_dqstats v2_stats;
+   struct v0_dqstats v0_stats;
    FILE *f;
    int ret = 0;
    struct stat st;
@@ -517,7 +525,7 @@ int kern_quota_format(fs_t *fs, int q_type) {
       fclose(f);
    }
    else if (stat("/proc/sys/fs/quota", &st) == 0) {
-      /* Either QF_VFSOLD or QF_VFSV0 */
+      /* Either QF_VFSOLD or QF_VFSV0 or QF_VFSV1 */
       int actfmt, retval;
       kernel_iface = IFACE_GENERIC;
       retval = quotactl(QCMD(Q_GETFMT, q_type), fs->device, 0, (void *) &actfmt);
@@ -532,30 +540,34 @@ int kern_quota_format(fs_t *fs, int q_type) {
 	    quota_format |= (1 << QF_VFSOLD);
 	 else if (actfmt == 2)  /* Q_GETFMT retval for QF_VFSV0 */
 	    quota_format |= (1 << QF_VFSV0);
-	 else
+	 else if (actfmt == 4)  /* Q_GETFMT retval for QF_VFSV1 */
+	    quota_format |= (1 << QF_VFSV1);
+	 else {
+            output_debug("Unknown Q_GETFMT: %d\n", actfmt);
 	    return QF_ERROR;
+         }
       }
       return ret;
    }
-   else if (quotactl(QCMD(Q_V2_GETSTATS, 0), NULL, 0, (void *) &v2_stats) >= 0) {
-      version = v2_stats.version;	/* Copy the version */
+   else if (quotactl(QCMD(Q_V0_GETSTATS, 0), NULL, 0, (void *) &v0_stats) >= 0) {
+      version = v0_stats.version;	/* Copy the version */
    }
    else {
       if (errno == ENOSYS || errno == ENOTSUP)	/* Quota not compiled? */
 	 return QF_ERROR;
       if (errno == EINVAL || errno == EFAULT || errno == EPERM) {	/* Old quota compiled? */
-	 /* RedHat 7.1 (2.4.2-2) newquota check 
-	  * Q_V2_GETSTATS in it's old place, Q_GETQUOTA in the new place
+	 /* RedHat 7.1 (2.4.2-2) newquota check
+	  * Q_V0_GETSTATS in it's old place, Q_GETQUOTA in the new place
 	  * (they haven't moved Q_GETSTATS to its new value) */
 	 int err_stat = 0;
 	 int err_quota = 0;
 	 char tmp[1024];         /* Just temporary buffer */
-	 
-	 if (quotactl(QCMD(Q_V1_GETSTATS, 0), NULL, 0, tmp))
+
+	 if (quotactl(QCMD(Q_OLD_GETSTATS, 0), NULL, 0, tmp))
 	    err_stat = errno;
-	 if (quotactl(QCMD(Q_V1_GETQUOTA, 0), "/dev/null", 0, tmp))
+	 if (quotactl(QCMD(Q_OLD_GETQUOTA, 0), "/dev/null", 0, tmp))
 	    err_quota = errno;
-	 
+
 	 /* On a RedHat 2.4.2-2 	we expect 0, EINVAL
 	  * On a 2.4.x 		we expect 0, ENOENT
 	  * On a 2.4.x-ac	we wont get here */
@@ -585,46 +597,25 @@ int kern_quota_format(fs_t *fs, int q_type) {
    return ret;
 }
 
-int xfs_reset_grace(quota_t *myquota, int grace_type) {
-   /*
-     This is a hack for XFS which doesn't allow setting
-     the current inode|block usage. 
-     Instead we temporarily raise the quota limits to
-     current usage + 1, and then restore the previous limits.
-     Either let it remain here, or rewrite the entire
-     handling of resetting grace times.
-   */
-
+int quota_reset_grace(quota_t *myquota, int grace_type) {
    quota_t temp_quota;
 
-   if (! QF_IS_XFS(quota_format)) return 1;
-
    memcpy(&temp_quota, myquota, sizeof(quota_t));
 
-   if (grace_type == GRACE_BLOCK) {
-      output_debug("xfs_reset_grace: BLOCK");
-      temp_quota.block_hard = temp_quota.block_soft = temp_quota.block_used + 1;
-      if (xfs_quota_set(&temp_quota) && xfs_quota_set(myquota)) {
-	 return 1;
-      }
-   }
-   else if (grace_type == GRACE_INODE) {
-      output_debug("xfs_reset_grace: INODE");
-      temp_quota.inode_hard = temp_quota.inode_soft = temp_quota.inode_used + 1;
-      if (xfs_quota_set(&temp_quota) && xfs_quota_set(myquota)) {
-	 return 1;
-      }
+   if (grace_type == GRACE_BLOCK)
+       temp_quota.block_hard = temp_quota.block_soft = BYTES_TO_BLOCKS(temp_quota.diskspace_used) + 1;
+   else
+       temp_quota.inode_hard = temp_quota.inode_soft = temp_quota.inode_used + 1;
+
+   if (QF_IS_XFS(quota_format)) {
+       if (xfs_quota_set(&temp_quota) && xfs_quota_set(myquota))
+           return 1;
    }
    else {
-      // We shouldn't get here
-      output_error("xfs_reset_grace(): wrong parameter for grace_type");
-      return 0;
+       if (quota_set(&temp_quota) && quota_set(myquota))
+           return 1;
    }
+
+   output_error("Cannot reset grace period!");
    return 0; // error, on success we return above
 }
-
-/* int quota_on (int q_type, char *device);
- * int quota_off (int q_type, char *device);
- */
-
-
