@@ -17,6 +17,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <errno.h>
 
 
 #include "quotatool.h"
@@ -41,10 +42,10 @@
 #define _PARSE_INODE 0x02
 /*
  * parse_commandline
- * read our args, parse them 
- * and return a struct of the data 
+ * read our args, parse them
+ * and return a struct of the data
  */
-argdata_t *parse_commandline (int argc, char **argv) 
+argdata_t *parse_commandline (int argc, char **argv)
 {
   argdata_t *data;
   extern char *optarg;
@@ -68,10 +69,10 @@ argdata_t *parse_commandline (int argc, char **argv)
   optarg = NULL;
   opterr = 0;
   done = fail = 0;
-  while ( ! done && ! fail ) {    
+  while ( ! done && ! fail ) {
     opt = getopt(argc, argv, OPTSTRING);
-    
-    if (opt > 0) 
+
+    if (opt > 0)
        output_debug ("option: '%c', argument: '%s'", opt, optarg);
 
     switch (opt) {
@@ -83,11 +84,11 @@ argdata_t *parse_commandline (int argc, char **argv)
     case 'h':
       output_help ();
       exit (0);
-      
+
     case 'V':
       output_version();
       exit (0);
-      
+
     case 'v':
       output_level++;
       break;
@@ -112,7 +113,7 @@ argdata_t *parse_commandline (int argc, char **argv)
 	data->id = optarg;
       }
       /* -u [-next-opt] */
-      else if ( ! argv[optind] || argv[optind][0] == '-' ) { 
+      else if ( ! argv[optind] || argv[optind][0] == '-' ) {
 	output_debug ("not mangling: NULL user");
 	data->id = NULL;
       }
@@ -123,11 +124,19 @@ argdata_t *parse_commandline (int argc, char **argv)
 	optind++;
       }
 #else
-      data->id = optarg;
+      if (optarg && ((data->block_grace || data->inode_grace) || (optarg[0] == '-'))) {
+          /* -u [-next-opt] */
+          optind--;
+          data->id = NULL;
+      }
+      else {
+          /* -u user */
+          data->id = optarg;
+      }
 #endif
       output_info ("using uid %s", data->id);
       break;
-      
+
     case 'g':   /* set groupname */
       if ( data->id_type ) {
 	output_error("Only one quota (user or group) can be set");
@@ -156,24 +165,20 @@ argdata_t *parse_commandline (int argc, char **argv)
       output_info ("using gid  %s", data->id);
       break;
 
-      
-
-    case 'b':   // set max blocks
-      output_info ("setting block limit");
+    case 'b':   // Work with block limits
+      output_info ("working with block limits");
       quota_type = _PARSE_BLOCK;
       break;
-      
-    case 'i':   // set max inodes
-      output_info ("setting inode limit");
+
+    case 'i':   // Work with inode limits
+      output_info ("working with inode limits");
       quota_type = _PARSE_INODE;
       break;
-      
 
-
-    case 'q':
+    case 'q': // soft limit
       switch ( quota_type ) {
       case _PARSE_UNDEF:
-	output_error ("must specify either block or inode");
+	output_error ("Must specify either block (-b) or inode (-i) before -q");
 	fail = 1;
 	break;
       case _PARSE_BLOCK:
@@ -188,12 +193,12 @@ argdata_t *parse_commandline (int argc, char **argv)
       }
       output_info ("setting soft limit to %s", optarg);
       break;
-      
 
-    case 'l':
+
+    case 'l': // hard limit
       switch ( quota_type ) {
       case _PARSE_UNDEF:
-	output_error ("must specify either block or inode");
+	output_error ("Must specify either block (-b) or inode (-i) before -l");
 	fail = 1;
 	break;
       case _PARSE_BLOCK:
@@ -210,12 +215,12 @@ argdata_t *parse_commandline (int argc, char **argv)
       break;
 
 
-      
-    case 't':
+
+    case 't': // set grace period
       data->id = NULL;
       switch ( quota_type ) {
       case _PARSE_UNDEF:
-	output_error ("must specify either block or inode");
+	output_error ("Must specify either block (-b) or inode (-i) before -t");
 	fail = 1;
 	break;
       case _PARSE_BLOCK:
@@ -228,14 +233,13 @@ argdata_t *parse_commandline (int argc, char **argv)
 	output_error ("Impossible error #42t: evacuate the building!");
 	break;
       }
-      output_info ("setting grace period to %s", optarg);
       break;
 
-      
-    case 'r':
+
+    case 'r': // reset grace time
       switch ( quota_type ) {
       case _PARSE_UNDEF:
-	output_error ("must specify either block or inode");
+	output_error ("Must specify either block (-b) or inode (-i) before -r");
 	fail = 1;
 	break;
       case _PARSE_BLOCK:
@@ -248,7 +252,6 @@ argdata_t *parse_commandline (int argc, char **argv)
 	output_error ("Impossible error #42r: evacuate the building!");
 	break;
       }
-      output_info ("resetting grace period");
       break;
 
     case 'd':
@@ -262,41 +265,44 @@ argdata_t *parse_commandline (int argc, char **argv)
     case ':':
       output_error ("Option '%c' requires an argument", optopt);
       break;
-      
+
     case '?':
       output_error ("Unrecognized option: '%c'", optopt);
-      
+
     default:
       output_help();
       fail = 1;
       break;
-      
-      
     }
   }
-  
 
   if ( fail ) {
     free (data);
     return NULL;
   }
-  
+
   if ( ! data->id_type ) {
     output_error ("Must specify either user or group quota");
     return NULL;
   }
-  
+
   if ( data->dump_info) {
      output_info("Option 'd' => just dumping quota-info for %s", data->id_type == QUOTA_USER ? "user" : "group");
   }
 
   /* the remaining arg is the filesystem */
   data->qfile = argv[optind];
-  if ( ! data->qfile ) {
+  if ( ! data->qfile || strlen(data->qfile) == 0) {
     output_error ("No filesystem specified");
     return NULL;
   }
 
+  /* remove trailing slash(es) except for / filesystem */
+  while (strlen(data->qfile) > 1) {
+    if (data->qfile[strlen(data->qfile) - 1] != '/') break;
+    data->qfile[strlen(data->qfile) - 1] = '\0';
+  }
+
   /* check for mixing -t with other options in the wrong way */
   if (data->block_grace || data->inode_grace) {
      if (data->block_hard || data->block_soft || data->inode_hard || data->inode_soft || data->id) {
@@ -304,20 +310,25 @@ argdata_t *parse_commandline (int argc, char **argv)
 	return NULL;
      }
   }
-  
+
+  /* check for mixing -r with other options in the wrong way */
+  if (data->block_reset || data->inode_reset) {
+      if (data->block_hard || data->block_soft || data->inode_hard || data->inode_soft) {
+          output_error("Wrong options for -r, please see manpage for usage instructions!");
+          return NULL;
+      }
+  }
+
   output_info ("using filesystem %s", data->qfile);
-  
+
   return data;
 }
 
-
-
-
 #define _PARSE_OP_ADD '+'
 #define _PARSE_OP_SUB '-'
 
 
-/* On aix MIN definition conflicts with MIN definition at 
+/* On aix MIN definition conflicts with MIN definition at
  * <include/sys/param.h>
  */
 #undef MIN
@@ -334,14 +345,14 @@ argdata_t *parse_commandline (int argc, char **argv)
  * understands seconds, minutes, hours, days, weeks, months
  * returns the number of seconds represented
  */
-time_t parse_timespan (time_t orig, char *string) 
+time_t parse_timespan (time_t orig, char *string)
 {
   char *cp;
   int count, unit;
   char op;
 
   op = '\0';
-  if ( ( *string == _PARSE_OP_ADD ) 
+  if ( ( *string == _PARSE_OP_ADD )
        || ( *string == _PARSE_OP_SUB ) ) {
     op = *string;
     string++;
@@ -352,10 +363,10 @@ time_t parse_timespan (time_t orig, char *string)
     output_error ("Invalid format: %s", string);
     return -1;
   }
-  
+
   /* remove whitespace */
   while ( strchr(WHITESPACE, *cp) ) cp++;
-  
+
 
   if ( ! strncasecmp(cp, "s", 1) ) {
     unit = SEC;
@@ -394,54 +405,69 @@ time_t parse_timespan (time_t orig, char *string)
 
 }
 
+#define KILO  1024ul
+#define MEGA  KILO * KILO
+#define GIGA  KILO * MEGA
+#define TERA  KILO * GIGA
 
-
-
-#define BYTE  1
-#define KILO  1024
-#define MEGA  1024*KILO
+#define KILO_10  1000ul
+#define MEGA_10  KILO_10 * KILO_10
+#define GIGA_10  KILO_10 * MEGA_10
+#define TERA_10  KILO_10 * GIGA_10
 /*
  * parse_size
- * understands Kb, Mb, bytes, and disk blocks
+ * understands Kb, Mb, Gb, Tb, bytes, and disk blocks
  * returns the number of bytes represented
  */
-u_int64_t parse_size (u_int64_t orig, char *string) {
+u_int64_t parse_size (u_int64_t orig, char *string, int parse_type) {
   char *cp;
-  u_int64_t blocks;
-  uint count, unit;
+  u_int64_t size, multiplier;
+  double count;
   char op;
 
   op = '\0';
-  if ( ( *string == _PARSE_OP_ADD ) 
+  if ( ( *string == _PARSE_OP_ADD )
        || ( *string == _PARSE_OP_SUB ) ) {
     op = *string;
     string++;
   }
 
   /* get the number */
-  count = strtol(string, &cp, 10);
-  if (cp == string) {      // No numeric argument
+  count = strtod(string, &cp);
+  if (count == 0 && string == cp) {      // No numeric argument
     return orig;
   }
 
+  /* negative sizes make no sense */
+  if (count<0)
+  {
+    exit (ERR_ARG);
+  }
+
   /* remove whitespace */
   while ( strchr(WHITESPACE, *cp) )  cp++;
 
   /* get the units */
   if ( ! strncasecmp(cp, "by", 2) ) {
-    unit = BYTE;
+    multiplier = 1;
   }
   else if ( ! strncasecmp(cp, "bl", 2) ) {
-    unit = BLOCK_SIZE;
+    multiplier = (parse_type == PARSE_BLOCKS ? BLOCK_SIZE : 1);
   }
   else if ( ! strncasecmp(cp, "k", 1) ) {
-    unit = KILO;
+    multiplier = (parse_type == PARSE_BLOCKS ? KILO : KILO_10);
   }
   else if ( ! strncasecmp(cp, "m", 1) ) {
-    unit = MEGA;
+    multiplier = (parse_type == PARSE_BLOCKS ? MEGA : MEGA_10);
   }
-  else {      // default to blocks
-    unit = BLOCK_SIZE;
+  else if ( ! strncasecmp(cp, "g", 1) ) {
+    multiplier = (parse_type == PARSE_BLOCKS ? GIGA : GIGA_10);
+  }
+  else if ( ! strncasecmp(cp, "t", 1) ) {
+    multiplier = (parse_type == PARSE_BLOCKS ? TERA : TERA_10);
+  }
+  else {      // default to blocks or inodes
+    multiplier = (parse_type == PARSE_BLOCKS ? BLOCK_SIZE : 1);
   }
 
   /* avoid a DIV0 */
@@ -449,16 +475,21 @@ u_int64_t parse_size (u_int64_t orig, char *string) {
     return 0;
   }
 
-  /* calculate disk blocks */
-  blocks = (u_int64_t) (((double) count*unit - 1) / BLOCK_SIZE) + 1;
+  /* calculate size (blocks or inodes) */
+  if (parse_type == PARSE_BLOCKS) {
+      size = (u_int64_t) (((double) count * multiplier - 1) / BLOCK_SIZE) + 1;
+  }
+  else { // inodes
+      size = (u_int64_t) (count * multiplier);
+  }
 
   switch (op) {
   case _PARSE_OP_ADD:
-    return  orig + blocks;
+    return  orig + size;
   case _PARSE_OP_SUB:
-    return  orig - blocks;
+    return  orig - size;
   default:
-    return blocks;
+    return size;
   }
 
 }
